(*******************************************************************
This file was generated automatically by the Mathematica front end.
It contains Initialization cells from a Notebook file, which
typically will have the same name as this file except ending in
".nb" instead of ".m".

This file is intended to be loaded into the Mathematica kernel using
the package loading commands Get or Needs.  Doing so is equivalent
to using the Evaluate Initialization Cells menu command in the front
end.

DO NOT EDIT THIS FILE.  This entire file is regenerated
automatically each time the parent Notebook file is saved in the
Mathematica front end.  Any changes you make to this file will be
overwritten.
***********************************************************************)





(* :Context: InvertCircle` *)

(* :Title: InvertCircle *)

(* :Author: Xah Lee *)

(* : Copyright 1996 by Xah Lee.
*)

(* :Summary:
This package is used primarily for
producing artistic images using the idea of multiple inversion
of circles.

This package exports the function InvertCircle.
InvertCircle[ Circle[{c1,c2}, s], Circle[{o1,o2}, r] ]
returns Circle[..] or Line[..] Graphics Primitive
that is the inversion of Circle[{c1,c2}, s]
with respect to Circle[{o1,o2}, r].
*)

(* :Keywords: graphics, geometry, transformation,
inversion, conformal map
*)

(* :Mathematica Version: 2.2 *)
(* :Package Version: 2, 1996 October. *)
(* :History:
Version 1.0, 1996/10. First Released.
Version 2, 2008-04, added the feature for the second argument to be a line. i.e. InvertCircle[Circle or Line,Line[{{a1 ,a2 },{b1 ,b2 }}]]
*)

(* :Sources: 
Mathematica Programming: 
Maeder, Roman: Programming In Mathematica, 2nd ed. chapter 1-3.
Smith, Cameron; Blachman, Nancy: The Mathematica Graphics Guidebook.
*)
		
(* :Warning: None. *)

(* :Limitations:  *)

(* :Discussion:
This package is used primarily for
producing artistic images using the idea of multiple inversion
of circles.

This package exports the function InvertCircle.
InvertCircle[ Circle[{c1,c2}, s], Circle[{o1,o2}, r] ]
returns Circle[..] or Line[..] Graphics Primitive
that is the inversion of Circle[{c1,c2}, s]
with respect to Circle[{o1,o2}, r].

Check the accompanying notebook
InvertCircleExamples.ma or InvertCircleExamples.nb
for complete documentation.

I wrote several other packages. They can be found on MathSource <http://www.MathSource.com/> or on my home page. If you have any problem with my packages, I'll be happy to help out.

Xah Lee, xah@xahlee.org
http://xahlee.org/

You should always be able to find me by searching the web.
*)

BeginPackage["InvertCircle`"]





InvertCircle::usage = "
	InvertCircle[ Circle[{c1,c2}, s], Circle[{o1,o2}, r] ]
	returns Circle[..] or Line[..] Graphics Primitive
	that is the inversion of Circle[{c1,c2}, s]
	with respect to Circle[{o1,o2}, r].
	InvertCircle[ Line[{{a1,a2},{b1,b2}}], Circle[{o1,o2}, r] ]
	returns the inversion of Line[{{a1,a2},{b1,b2}}].
	InvertCircle[ list1, listOfCircles ]
	returns the inversion of list1 with respect to listOfCircles.
	list1 must consists of Circle or Line Graphics Primitive.
	Usage Examples: InvertCircle[ Line[{{1,-5},{1,5}}], Circle[{0,0},2] ];
	InvertCircle[ Circle[{1,1},1], Circle[{0,0},2] ];
	InvertCircle[{Circle[{1,1},1], Line[{{1,-5},{1,5}}]}, {Circle[{0,0},2]}]";

InvertCircleNest::usage = "
InvertCircleNest[ list1, list2, n, (listOfColors) ]
returns a list of Circle that is the result of inverting
list1 of circles with respect to list2 of circles, nested n times.
It is equivalent to Nest[ InvertCircle[ #, list2]&, list1, n ]
with added color directive.
The optional listOfColors is a list of color directives.
listOfColors must have the same number of elements as of list1.
Default listofColors is evenly distributed Hue.
Usage Example:
Show@ Graphics@
InvertCircleNest[
{Circle[{-1,0},1], Circle[{1,0},1]},
{Circle[{-1,0},1], Circle[{1,0},1]},
4
]";

InvertCircleNestList::usage = "
InvertCircleNestList[ list1, list2, n , (colorList)]
gives a result that is equivalent to
Table[ InvertCircleNest[list1, list2, i], {i,0,n}].
See ?InvertCircleNest.";

Begin["`Private`"]



InvertCircle[ list1_List , list2:{_Circle..}] :=
Outer[ InvertCircle[#1,#2]&, N@ list1, N@ list2]



InvertCircle[Circle[{c1_,c2_},s_],Circle[{o1_,o2_},r_]]:=
  Module[{q1,q2,oc},(*oc is distance from O to C*)oc=
      Sqrt[Plus@@(({c1,c2}-{o1,o2})^2)];
    Which[(*same circle*)(N@oc\[Equal]0.)&&(N@r\[Equal]N@s),
      Circle[{c1,c2},s],(*inversion point on circle*)N@oc\[Equal]N@s,
      Line[10000 {-{-Last@#,First@#},{-Last@#,First@#}}+{#,#}/2. (r/s)^2+{{o1,
                  o2},{o1,o2}}]&@({c1,c2}-{o1,
              o2}),(*both circles share the same center*)N@oc\[Equal]0.,
      Circle[{c1,c2},r^2/s],(*"normal" cases*)True,
      Circle[({c1,c2}-{o1,o2})/oc*(oc r^2/(oc^2-s^2))+{o1,o2},
        Abs[r^2 s/(oc^2-s^2)]]]]

InvertCircle[Line[{{a1_,a2_},{b1_,b2_}}],Circle[{o1_,o2_},r_]]:=
  Module[{lineUnitV,cP},
    lineUnitV=({b1,b2}-{a1,a2})/Sqrt[Plus@@(({b1,b2}-{a1,a2})^2)];
    cP={a1,a2}+({o1,o2}-{a1,a2}).lineUnitV*lineUnitV;
    If[N@cP\[Equal]N@{o1,o2},Line[{{a1,a2},{b1,b2}}],
      Circle[(#+{o1,o2})/2,
            Sqrt[Plus@@((#-{o1,o2})^2)]/
              2]&@((cP-{o1,o2})/(Plus@@((cP-{o1,o2})^2)) r^2+{o1,o2})
      (*this is cQ,the inverse of cP*)]]

InvertCircle[Circle[{c1_,c2_},s_],Line[{{a1_,a2_},{b1_,b2_}}]]:=
  Circle[reflect[{c1,c2},Line[{{a1,a2},{b1,b2}}]],s]

InvertCircle[Line[{{c1_,c2_},{d1_,d2_}}],Line[{{a1_,a2_},{b1_,b2_}}]]:=
  Line[{{c1,c2},{d1,d2}}]/.{x_,y_}\[Rule]
      reflect[{x,y},Line[{{a1,a2},{b1,b2}}]]









reflect[{x_,y_},Line[{{a1_,a2_},{b1_,b2_}}]]:=
  Module[{rotV},rotV=({b1-a1,b2-a2})//(#/Sqrt@(Plus@@(#^2))&);
    rotate[({x,y}-{a1,a2}),
          rotV*{1,-1}]//(#*{1,-1}+{a1,-a2}&)//(rotate[#,rotV]&)]
rotate[{a_,b_},{c_,d_}]:={a*c-b*d,b*c+a*d}



InvertCircleNest[list1:{((_Circle)|(_Line))..},list2:{_Circle..},n_Integer,
    colorList_List:{Automatic}]:=
  Module[{colorList2},
    colorList2=
      N@If[colorList==={Automatic},
          Table[Hue[i],{i,0,1-(1/(Length@list1)/2),1/(Length@list1)}],
          colorList];
    Transpose[{colorList2,Nest[InvertCircle[#,list2]&,list1,n]}]]

InvertCircleNestList[list1:{((_Circle)|(_Line))..},list2:{_Circle..},
    n_Integer,colorList_List:{Automatic}]:=
  Module[{colorList2},
    colorList2=
      N@If[colorList==={Automatic},
          Table[Hue[i],{i,0,1-(1/(Length@list1)/2),1/(Length@list1)}],
          colorList];
    Transpose[{colorList2,#}]&/@NestList[InvertCircle[#,list2]&,list1,n]]

End[]

Protect[InvertCircle, InvertCircleNest, InvertCircleNestList]

EndPackage[]